import pytest

from blue_brain_token_fetch.job import InterruptionStack

SERVICE_CONFIG = "./tests/tests_data/service_keycloak_config.yaml"
REGULAR_CONFIG_WITH_CLIENT_PWD = "./tests/tests_data/regular_keycloak_config_with_password.yaml"
REGULAR_CONFIG = "./tests/tests_data/regular_keycloak_config.yaml"


def pytest_addoption(parser):
    parser.addoption("--regular_username", action="store", required=True)
    parser.addoption("--regular_password", action="store", required=True)
    parser.addoption("--service_username", action="store", required=True)
    parser.addoption("--service_password", action="store", required=True)


@pytest.fixture(scope="session")
def regular_username(pytestconfig):
    return pytestconfig.getoption("regular_username")


@pytest.fixture(scope="session")
def regular_password(pytestconfig):
    return pytestconfig.getoption("regular_password")


@pytest.fixture(scope="session")
def service_username(pytestconfig):
    return pytestconfig.getoption("service_username")


@pytest.fixture(scope="session")
def service_password(pytestconfig):
    return pytestconfig.getoption("service_password")


def pytest_sessionfinish(session, exitstatus):
    InterruptionStack.callable_stack()



from keycloak import KeycloakAuthenticationError

from blue_brain_token_fetch.token_fetcher_service import TokenFetcherService
from tests.conftest import SERVICE_CONFIG

import pytest


def test_service_account(service_username, service_password):
    test = TokenFetcherService(
        keycloak_config_file=SERVICE_CONFIG, username=service_username, password=service_password
    )

    token = test.get_access_token()
    duration = test.get_access_token_duration()
    assert token is not None
    assert duration is not None


def test_invalid_credentials(service_username):
    with pytest.raises(KeycloakAuthenticationError):
        TokenFetcherService(
            service_username,
            "password",
            SERVICE_CONFIG,
        )



from pathlib import Path
from click.testing import CliRunner

from blue_brain_token_fetch.nexus_token_fetch import token_fetcher

TEST_PATH = Path(Path(__file__).parent.parent)


def test_token_fetcher_cli():

    username = "username"
    password = "password"

    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(
            token_fetcher,
            [
                "--username",
                username,
                "--password",
                password,
            ],
        )
        assert result.exit_code == 1



import pytest
from blue_brain_token_fetch.duration_converter import (
    convert_duration_to_sec,
    _convert_string_to_time_unit,
)


def test_convert_duration_to_sec():

    duration = "3"
    assert convert_duration_to_sec(duration) == 3.0

    duration = "10seconds"
    assert convert_duration_to_sec(duration) == 10.0

    duration = "0.5h"
    assert convert_duration_to_sec(duration) == 1800.0

    # Errors:

    duration = "-0.5sec"
    with pytest.raises(ValueError) as e:
        convert_duration_to_sec(duration)
    assert "The number detected in the input duration need to be positive" in str(
        e.value
    )

    duration = "time"
    with pytest.raises(TypeError) as e:
        convert_duration_to_sec(duration)
    assert "not of the form:\n'{float > 0}{{eventual unit of time}}'" in str(e.value)

    duration = "5weeks"
    with pytest.raises(ValueError) as e:
        convert_duration_to_sec(duration)
    assert (
        "does not correspond to these time units : seconds, minutes, hours, days"
        in str(e.value)
    )


def test_convert_string_to_time_unit():

    time_unit = "s"
    assert _convert_string_to_time_unit(time_unit) == 1

    time_unit = "d"
    assert _convert_string_to_time_unit(time_unit) == 86400

    time_unit = "others"
    with pytest.raises(ValueError) as e:
        _convert_string_to_time_unit(time_unit)
    assert (
        "does not correspond to these time units : seconds, minutes, hours, days"
        in str(e.value)
    )



"""
This CLI allows the fetching and the automatic refreshing of the Nexus token using
Keycloak.
Its value can be written periodically in a file whose path is given in input or be
displayed on the console output as desired.
For more information about Nexus, see https://bluebrainnexus.io/
"""
import os
import time
import logging
import click

from blue_brain_token_fetch.token_fetcher_base import TokenFetcherBase
from blue_brain_token_fetch.token_fetcher_user import TokenFetcherUser
from blue_brain_token_fetch.duration_converter import convert_duration_to_sec
from blue_brain_token_fetch import __version__
from blue_brain_token_fetch.token_fetcher_service import TokenFetcherService

L = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


class HiddenPassword(object):
    def __init__(self, password=""):
        self.password = password

    def __str__(self):
        return "*" * 4


@click.command()
@click.version_option(__version__)
@click.option(
    "--username",
    prompt=True,
    default=lambda: os.environ.get("USER", ""),
    show_default=f"Username detected by whoami : {os.environ.get('USER', '')}",
    help="Username to request the access token",
)
@click.option(
    "--password",
    prompt=True,
    hide_input=True,
    help="Password to request the access token",
)
@click.option(
    "--output",
    "-o",
    is_flag=True,
    flag_value=False,
    default=True,
    help=(
        "Flag option allowing for 3 distinct outputs:\t\t\t"
        "- {not_given} : By default the fetched token will be written in the file "
        f"located at {TokenFetcherBase.DEFAULT_TOKEN_FILEPATH_LABEL},\t\t\t\t"
        "- {-o/--output} : Providing only the flag will print the token on the "
        "console output,\t"
        "- {-o/--output} {PATH}: If a value (argument 'path') is given as a file path, "
        "the token will be written in this file location,\t"
        "Note: The output file containing the token will have owner read/write access"
    ),
)
@click.argument(
    "path",
    required=False,
    type=click.Path(),
)
@click.option(
    "--refresh-period",
    "-rp",
    default="15",
    help=(
        "Duration of the period between which the token will be written in the file. "
        "It can be expressed as number of seconds or by using time unit : "
        "'{float}{time unit}'.\t\t\t\t "
        "Available time unit are :\t\t\t\t\t "
        "- ['s', 'sec', 'secs', 'second', 'seconds'] for seconds, \t\t\t\t\t\t"
        "- ['m', 'min', 'mins', 'minute', 'minutes'] for minutes, \t\t\t\t\t\t\t"
        "- ['h', 'hr', 'hrs', 'hour', 'hours'] for hours, "
        "- ['d', 'day', 'days'] for days. \t\t\t"
        "Ex: '-rp 30' '-rp 30sec', '-rp 0.5min', '-rp 0.1hour'"
    ),
)
@click.option(
    "--timeout",
    "-to",
    help=(
        "Duration corresponding to the life span to be applied to the application "
        "before it is stopped. It can be expressed as number of seconds or by using "
        "time unit : '{float}{time unit}'.\t\t\t\t"
        "Available time unit are :\t\t\t\t\t "
        "- ['s', 'sec', 'secs', 'second', 'seconds'] for seconds, \t\t\t\t\t\t"
        "- ['m', 'min', 'mins', 'minute', 'minutes'] for minutes, \t\t\t\t\t\t\t"
        "- ['h', 'hr', 'hrs', 'hour', 'hours'] for hours, "
        "- ['d', 'day', 'days'] for days. \t\t\t"
        "Ex: '-rp 30' '-rp 30sec', '-rp 0.5min', '-rp 0.1hour'"
    ),
)
@click.option(
    "--keycloak-config-file",
    "-kcf",
    type=click.Path(exists=True),
    help=(
        "The path to the yaml file containing the configuration to create the "
        "keycloak instance. If not provided, it will search in your $HOME directory "
        f"for a '{TokenFetcherBase.DEFAULT_TOKEN_FILEPATH}' file "
        "containing the keycloak configuration.\t\tIf this file does not exist or the "
        "configuration inside is wrong, the configuration will be prompt in the "
        "console output and saved in the $HOME directory under the name: "
        f"'{TokenFetcherBase.DEFAULT_TOKEN_FILEPATH}'."
    ),
)
@click.option("--verbose", "-v", count=True)
@click.option("--service", "-s", count=False,
              help="Whether the account is a service account or not")
def token_fetcher(
    username,
    password,
    output,
    path,
    refresh_period,
    timeout,
    keycloak_config_file,
    verbose,
    service
):
    """
    As a first step it fetches the Nexus access token using Keycloak and the
    username/password values.
    Then it writes it in the given file or displayed it on the console output every
    given 'refresh_period'.\n
    Finally, the process is stopped when the duration reach the value given by the
    'timeout' argument.
    """
    L.setLevel((logging.WARNING, logging.INFO, logging.DEBUG)[min(verbose, 2)])

    if isinstance(password, HiddenPassword):
        password = password.password

    try:
        refresh_period = convert_duration_to_sec(refresh_period)
    except Exception as e:
        L.error(f"Error: {e}")
        exit(1)

    init_cls = TokenFetcherUser if not service else TokenFetcherService
    try:
        my_token_fetcher: TokenFetcherBase = init_cls(username, password, keycloak_config_file)
    except Exception as e:
        L.error(f"Error: {e}")
        exit(1)

    start_time = time.time()
    flag_rp = 0
    flag_to = 0
    flag_console = 0
    while True:

        my_access_token = my_token_fetcher.get_access_token()

        # if refresh period is superior to half of access token duration
        if (
            flag_rp == 0
            and my_token_fetcher.get_access_token_duration() // 2 < refresh_period
        ):
            flag_rp += 1
            L.info(
                f"The refresh period (= {refresh_period} seconds) is greater than the "
                "value of half the access token life span "
                f"(= {my_token_fetcher.get_access_token_duration() // 2:g} seconds)). The "
                "refresh period thus becomes equal to : "
                f"{my_token_fetcher.get_access_token_duration() // 2:g} seconds)."
            )
            refresh_period = my_token_fetcher.get_access_token_duration() // 2

        if timeout:
            if flag_to == 0:
                flag_to += 1
                try:
                    timeout = convert_duration_to_sec(timeout)
                except Exception as e:
                    L.error(f"Error: {e}")
                    exit(1)

                if timeout < refresh_period:
                    L.info(
                        f"The timeout argument (= {timeout:g} seconds) is shorter "
                        f"than the refresh period (= {refresh_period:g} seconds). The "
                        "app will shut down after one refresh period."
                    )

        if path is None and not output:
            if flag_console == 0:
                flag_console += 1
                print(
                    "\n===================== Nexus Token =====================\n\n"
                    f"This access token will be refreshed every {refresh_period:g} "
                    f"seconds:\n\n"
                )
                print(f"{my_access_token}")
            else:
                print(f"\x1B[7A{my_access_token}")
        else:

            output_path = path or TokenFetcherBase.DEFAULT_TOKEN_FILEPATH
            os.makedirs(os.path.dirname(output_path), exist_ok=True)

            L.info(
                f"The token will be written in the file '{output_path}' every "
                f"{refresh_period:g} seconds.\r"
            )
            with open(output_path, "w") as f:
                f.write(my_access_token)
            os.chmod(output_path, 0o0600)

        time.sleep(refresh_period)

        if timeout:
            if time.time() > (start_time + timeout):
                L.info("\n> Timeout reached, successfully exit.")
                exit(1)


def start():
    token_fetcher(obj={})


if __name__ == "__main__":
    start()



"""This class allows the fetching and the automatic refreshing of the Nexus token using
Keycloak.
It contains 2 public methods to get a fresh Nexus access token and to get its life
duration.
For more information about Nexus, see https://bluebrainnexus.io/
"""
from typing import Tuple, Dict, Callable

from keycloak import KeycloakOpenID

from blue_brain_token_fetch.job import Job
from blue_brain_token_fetch.token_fetcher_base import TokenFetcherBase


class TokenFetcherUser(TokenFetcherBase):

    _refresh_token = None
    _refresh_token_duration = None

    @classmethod
    def config_keys(cls) -> Dict[str, bool]:
        return {
            "SERVER_URL": True,
            "CLIENT_ID": True,
            "REALM_NAME": True,
            "CLIENT_PASSWORD": False
        }

    def get_access_token(self):
        return self._keycloak_openid.refresh_token(self._refresh_token)["access_token"]

    def _refresh_perpetually(self) -> Callable:
        """
        Launch the thread encapsulating '_periodically_refresh_token()'.
        """
        return Job.schedule(
            self._refresh_refresh_token, self._refresh_token_duration / 2,
            "stopping refreshing of refresh token"
        )

    def _refresh_refresh_token(self):
        """
        Periodically refresh the 'refresh token' every half of its life duration.
        """
        print("Refresh token thingy")
        self._refresh_token = self._keycloak_openid.refresh_token(self._refresh_token)["refresh_token"]

    def _get_keycloak_instance_and_payload(
            self, username, password, keycloak_config
    ) -> Tuple[KeycloakOpenID, Dict]:

        instance = KeycloakOpenID(
            server_url=keycloak_config["SERVER_URL"],
            client_id=keycloak_config["CLIENT_ID"],
            client_secret_key=keycloak_config.get("CLIENT_PASSWORD"),
            realm_name=keycloak_config["REALM_NAME"],
        )

        payload = instance.token(username, password)
        self._refresh_token = payload["refresh_token"]
        self._refresh_token_duration = payload["refresh_expires_in"]
        return instance, payload



from keycloak import KeycloakAuthenticationError, KeycloakPostError, KeycloakConnectionError

from blue_brain_token_fetch.token_fetcher_user import TokenFetcherUser
from tests.conftest import REGULAR_CONFIG

import pytest


def test_regular_account(regular_username, regular_password):
    test = TokenFetcherUser(
        keycloak_config_file=REGULAR_CONFIG, username=regular_username, password=regular_password
    )

    token = test.get_access_token()
    duration = test.get_access_token_duration()
    assert token is not None
    assert duration is not None


def test_invalid_credentials(regular_username):
    with pytest.raises(KeycloakAuthenticationError):
        TokenFetcherUser(
            regular_username,
            "password",
            REGULAR_CONFIG,
        )


def test_invalid_config_file():
    with pytest.raises(ValueError):
        TokenFetcherUser(
            "username",
            "password",
            "./tests/tests_data/empty_keycloak_config.yaml",
        )
    with pytest.raises(KeyError):  # missing client id
        TokenFetcherUser(
            "username",
            "password",
            "./tests/tests_data/service_keycloak_config.yaml",
        )
    with pytest.raises(KeycloakPostError):  # Invalid realm
        TokenFetcherUser(
            "username",
            "password",
            "./tests/tests_data/regular_keycloak_config_invalid_realm.yaml",
        )
    with pytest.raises(KeycloakConnectionError):  # Invalid server url
        TokenFetcherUser(
            "username",
            "password",
            "./tests/tests_data/regular_keycloak_config_invalid_server_url.yaml",
        )
    with pytest.raises(KeycloakAuthenticationError):  # Invalid client id
        TokenFetcherUser(
            "username",
            "password",
            "./tests/tests_data/regular_keycloak_config_invalid_client_id.yaml",
        )


def test_non_existing_config_file():
    with pytest.raises(FileNotFoundError):
        TokenFetcherUser(
            "username",
            "password",
            "non existent config file",
        )



import threading
import signal
from datetime import timedelta
from typing import Callable, List


class InterruptionStack:
    stack: List[Callable] = []

    @classmethod
    def callable_stack(cls):
        for c in InterruptionStack.stack:
            c()


class Job(threading.Thread):
    def __init__(self, interval, execute):
        threading.Thread.__init__(self)
        self.daemon = False
        self.stopped = threading.Event()
        self.interval = interval
        self.execute = execute

    def stop(self):
        self.stopped.set()
        # self.join()

    def run(self):
        while not self.stopped.wait(self.interval.total_seconds()):
            self.execute()

    @staticmethod
    def schedule(execute, interval, interruption_str="") -> Callable:
        """Returns handle to be capable of manually interrupt the job"""

        job = Job(interval=timedelta(seconds=interval), execute=execute)

        def test():
            print(f"Program killed: {interruption_str}")
            job.stop()

        InterruptionStack.stack.append(test)
        signal.signal(signal.SIGTERM, lambda a, b: InterruptionStack.callable_stack())
        signal.signal(signal.SIGINT, lambda a, b: InterruptionStack.callable_stack())

        job.start()

        return lambda: job.stop()



"""This class allows the fetching and the automatic refreshing of the Nexus token using
Keycloak.
It contains 2 public methods to get a fresh Nexus access token and to get its life
duration.
For more information about Nexus, see https://bluebrainnexus.io/
"""
import os
from abc import abstractmethod, ABC
from typing import Dict, Tuple, List

import getpass
import logging
import yaml

from keycloak import KeycloakOpenID
from keycloak.exceptions import KeycloakError, KeycloakAuthenticationError

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


class TokenFetcherBase(ABC):
    """
    A class to represent a Token Fetcher.

    Attributes
    ----------
    username : str
        gaspard identifier to access the Nexus token
    password : str
        gaspard identifier to access the Nexus token
    keycloak_config_file : Path
        Path of the keycloak configuration file

    Methods
    -------
    get_access_token():
        Return a fresh Nexus access token.
    get_access_token_duration():
        Return the access token life duration.
    """

    DEFAULT_CONFIG_FILENAME = "keycloack_config.yaml"
    DEFAULT_TOKEN_FILENAME = "Token"
    DEFAULT_SUBDIRECTORY = ".token_fetch"

    DEFAULT_DIRECTORY = os.path.join(os.environ.get('HOME', ''), DEFAULT_SUBDIRECTORY)
    DEFAULT_CONFIG_FILEPATH = os.path.join(DEFAULT_DIRECTORY, DEFAULT_CONFIG_FILENAME)
    DEFAULT_TOKEN_FILEPATH = os.path.join(DEFAULT_DIRECTORY, DEFAULT_TOKEN_FILENAME)

    DEFAULT_DIRECTORY_LABEL = os.path.join("$HOME", DEFAULT_SUBDIRECTORY)
    DEFAULT_CONFIG_FILEPATH_LABEL = os.path.join(DEFAULT_DIRECTORY, DEFAULT_CONFIG_FILENAME)
    DEFAULT_TOKEN_FILEPATH_LABEL = os.path.join(DEFAULT_DIRECTORY, DEFAULT_TOKEN_FILENAME)

    def __init__(self, username=None, password=None, keycloak_config_file=None):
        """
        Constructs all the necessary attributes for the TokenFetcher object. After
        that, call the appropriate method launching the perpetual token refreshing
        depending on whether identifiers have been given or not.

        Parameters
        ----------
            username : str
                gaspard identifier to access the Nexus token
            password : str
                gaspard identifier to access the Nexus token
            keycloak_config_file : str (file path)
                Path of the keycloak configuration file
        """

        username, password = self._get_credentials(username, password)
        keycloak_config = self._load_keycloak_config(keycloak_config_file)

        try:
            self._keycloak_openid, self._keycloak_payload = self._get_keycloak_instance_and_payload(
                username, password, keycloak_config
            )

            self._interrupt_callback = self._refresh_perpetually()

            del password

        except (KeycloakAuthenticationError, KeycloakError) as error:
            logger.error("⚠️ %s. Authentication failed, %s", error.__class__.__name__, error)
            raise error

    @abstractmethod
    def _refresh_perpetually(self):
        ...

    @classmethod
    @abstractmethod
    def config_keys(cls) -> Dict[str, bool]:
        """
        Keys to be retrieved from the configuration file, and whether they are mandatory or not
        """
        pass

    @classmethod
    def _load_keycloak_config(cls, keycloak_config_file=None):

        if keycloak_config_file:
            file_name = keycloak_config_file
        else:
            file_name = TokenFetcherBase.DEFAULT_TOKEN_FILEPATH
            logger.info('Keycloak configuration file found : %s', file_name)

        try:
            if not os.path.exists(file_name):
                raise FileNotFoundError(f"⚠️  FileNotFoundError. Cannot find file {file_name}")
            try:
                with open(file_name) as config_file:
                    config_content = yaml.safe_load(config_file.read().strip())
            except OSError as error:
                raise OSError(f"⚠️  OSError. {error}") from error

            if config_content is None:
                raise ValueError("⚠️  Keycloak configuration file is empty")

            try:
                [config_content[key] for key, mandatory in cls.config_keys().items() if mandatory]
            except KeyError as error:
                raise KeyError(
                    f"⚠️  KeyError {error}. Mandatory keys in the keycloak configuration file are"
                    f" {cls.config_keys()}"
                ) from error

            return config_content

        except Exception as e:

            logger.info(
                "Error when extracting the keycloak configuration from %s. %s.", file_name, e
            )

            if keycloak_config_file is not None:
                raise e

            logger.info("This latter will be reset with the new given configuration:")

            # only accept input for default file, not specified keycloak config file
            config_dict = dict(
                (key, input(f"Enter the {key.lower().replace('_', ' ')}:") or None)
                for key in cls.config_keys()
            )

            os.makedirs(os.path.dirname(file_name), exist_ok=True)

            with open(file_name, "w") as f:
                yaml.dump(config_dict, f)

            logger.info(
                "This configuration will be saved in the file %s that will be reused next time.",
                file_name
            )

            return config_dict

    def get_access_token_duration(self):
        return self._keycloak_payload["expires_in"]

    @abstractmethod
    def get_access_token(self):
        pass

    @abstractmethod
    def _get_keycloak_instance_and_payload(
            self, username, password, keycloak_config
    ) -> Tuple[KeycloakOpenID, Dict]:
        pass

    @staticmethod
    def _get_credentials(username: str, password: str) -> Tuple[str, str]:
        if username and password:
            return username, password

        detected_user = getpass.getuser()
        username = input(f"Username [{detected_user}]: ")
        if not username:
            username = detected_user
        password = getpass.getpass()

        return username, password



import os
from typing import Type
from contextlib import nullcontext as does_not_raise

import pytest

from blue_brain_token_fetch.token_fetcher_user import TokenFetcherUser
from blue_brain_token_fetch.token_fetcher_base import TokenFetcherBase
from blue_brain_token_fetch.token_fetcher_service import TokenFetcherService
from tests.conftest import SERVICE_CONFIG, REGULAR_CONFIG, REGULAR_CONFIG_WITH_CLIENT_PWD


@pytest.mark.parametrize("class_to_use, keycloak_config_filepath, expected_size, exception", [
    pytest.param(
        TokenFetcherService, SERVICE_CONFIG, 2, does_not_raise(), id="sa"
    ),
    pytest.param(
        TokenFetcherUser, REGULAR_CONFIG, 3, does_not_raise(), id="non_sa"
    ),
    pytest.param(
        TokenFetcherUser, REGULAR_CONFIG_WITH_CLIENT_PWD, 4, does_not_raise(), id="non_sa_pwd"
    )
])
def test_config_file(
        class_to_use: Type[TokenFetcherBase], keycloak_config_filepath, expected_size, exception
):
    with exception:
        keycloak_config = class_to_use._load_keycloak_config(keycloak_config_filepath)
        assert len(keycloak_config) == expected_size


def test_non_existing_default_file(monkeypatch):
    # When the file at the location determined by TokenFetcherBase.DEFAULT_TOKEN_FILEPATH
    # cannot be loaded (ex: doesn't exist, but the directory does), the user will be
    # prompted to input the configuration values, and the file will be created with them.

    TokenFetcherBase.DEFAULT_TOKEN_FILEPATH = "./tests/tests_data/non_existing_file.yaml"

    assert not os.path.exists(TokenFetcherBase.DEFAULT_TOKEN_FILEPATH)

    prompt_to_response = {
        "Enter the server url:": "a",
        "Enter the client id:": "b",
        "Enter the realm name:": "c",
        "Enter the client password:": None
    }

    def input_response(t):
        return prompt_to_response[t]

    monkeypatch.setattr('builtins.input', input_response)

    ee = TokenFetcherUser._load_keycloak_config(None)

    assert ee == dict(zip(
        TokenFetcherUser.config_keys().keys(),
        prompt_to_response.values()
    ))

    assert os.path.exists(TokenFetcherBase.DEFAULT_TOKEN_FILEPATH)
    os.remove(TokenFetcherBase.DEFAULT_TOKEN_FILEPATH)



"""This class allows the fetching and the automatic refreshing of the Nexus token using
Keycloak.
It contains 2 public methods to get a fresh Nexus access token and to get its life
duration.
For more information about Nexus, see https://bluebrainnexus.io/
"""
from typing import Tuple, Dict

from keycloak import KeycloakOpenID
from blue_brain_token_fetch.token_fetcher_base import TokenFetcherBase


class TokenFetcherService(TokenFetcherBase):

    def get_access_token(self):
        return self._keycloak_openid.token(grant_type="client_credentials")["access_token"]

    @classmethod
    def config_keys(cls) -> Dict[str, bool]:
        return {"SERVER_URL": True, "REALM_NAME": True}

    def _refresh_perpetually(self):
        pass

    def _get_keycloak_instance_and_payload(
            self, username, password, keycloak_config
    ) -> Tuple[KeycloakOpenID, Dict]:

        instance = KeycloakOpenID(
            server_url=keycloak_config["SERVER_URL"],
            realm_name=keycloak_config["REALM_NAME"],
            client_id=username,
            client_secret_key=password,
        )
        payload = instance.token(grant_type="client_credentials")

        return instance, payload



from setuptools import setup, find_packages

setup(
    name="blue_brain_token_fetch",
    author="Blue Brain Project, EPFL",
    use_scm_version=True,
    version="v2.0.1",
    setup_requires=["setuptools_scm"],
    description="Package to perform fetching and automatic refreshing of the Nexus "
    "access token using Keycloak. Using the CLI you can choose to either have its "
    "value periodically written in the file whose path is given in input or either "
    "have it periodically printed on the console output.",
    url="https://github.com/BlueBrain/bbp-token-fetch",
    license="Apache-2.0",
    python_requires=">=3.6.0",
    install_requires=[
        "click>=7.0",
        "python-keycloak>=0.24.0",
        "PyYAML>=5.3.1",
    ],
    extras_require={
        "dev": [
            "pytest>=4.3.0",
            "pytest-cov==4.1.0",
            "pycodestyle==2.11.1",
            "pylint==3.0.2"
        ],
    },
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "blue-brain-token-fetch=blue_brain_token_fetch.nexus_token_fetch:start"
        ]
    },
)



"""These functions allow to return any given duration of the form :
'{float > 0}{eventual unit of time (string)}' in seconds. Time units covered are :
seconds, minutes, hours and days.
"""
import re

m = 60
h = m * 60
d = h * 24


def convert_duration_to_sec(duration):
    """
    Take in input a duration (string) '{float > 0}{eventual unit of time}' and
    return its value in seconds.

    Parameters:
        duration : (string) of the form '{float > 0}{eventual unit of time}'.

    Returns:
        value in seconds of the input duration (float).
    """
    coefficient = 1
    re_expression = (
        r"^(-?(?:\d+)?\.?\d+) *(seconds?|secs?|s|minutes?|mins?|m|hours?|hrs?|h|days?"
        "|d|weeks?|w|years?|yrs?|y)?|d$"
    )

    match = re.match(re_expression, duration, re.M | re.I)

    if match:
        if match.group(2):
            coefficient = _convert_string_to_time_unit(match.group(2))
        if float(match.group(1)) > 0:
            return float(match.group(1)) * coefficient

        raise ValueError(
            "The number detected in the input duration need to be positive."
        )

    raise TypeError(
        f"Input duration '{duration}' is not of the form:\n'{{float > 0}}"
        "{{eventual unit of time}}'"
    )


def _convert_string_to_time_unit(time_unit):

    seconds = ["s", "sec", "secs", "second", "seconds"]
    minutes = ["m", "min", "mins", "minute", "minutes"]
    hours = ["h", "hr", "hrs", "hour", "hours"]
    days = ["d", "day", "days"]

    if time_unit in seconds:
        return 1
    elif time_unit in minutes:
        return m
    elif time_unit in hours:
        return h
    elif time_unit in days:
        return d
    else:
        raise ValueError(
            f"Input time_unit {time_unit} does not correspond to these time units : "
            "seconds, minutes, hours, days."
        )
